/* ******************************************************************************

	FILE:			Minimal.cpp
	AUTHOR:			Jean-Sbastien Perrier

	DESCRIPTION:	Screen saver application.

	$History::                                                                  $

****************************************************************************** */

// Includes ****************************************************************** //
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "SSMinimalApp.h"
#include "SSUtils.h"

// Defines ******************************************************************* //
#define REGSTR_PATH_CONFIG  ("Software\\FlipcodeStuff\\SSMinimalApp")

// Types ********************************************************************* //

// Consts ******************************************************************** //

// Globals ******************************************************************* //

// Static Members Storage Point ********************************************** //

// Function Prototypes ******************************************************* //

// Module External Functions ************************************************* //

// Module Internal Functions ************************************************* //


/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	SSMinimalApp
// DESCRIPTION:	Initialise the application's internal values. It is important
//				that the constructor initialise all the parameters to
//				a default value because they will get propagated in the
//				dialog box if the configuration has not been saved in the
//				registry.
// PARAMETERS:	(in) a_hInstance	-	Instance of the application.
// RETURN VALUE: None
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
SSMinimalApp::SSMinimalApp(HINSTANCE a_hInstance) : 
	SSBaseApp(a_hInstance), 
	m_bRotate(TRUE),
	m_bFlash(TRUE),
	m_pTexture(NULL)
{
	// Set the default texture file name
	SetTextureFile("SSTexture.bmp");
	// Setup initial position to 0 degrees.
	memset(m_afRot, 0, sizeof(float)*3);
	// Setup rotation speeds on the 3 axis in degrees/sec.
	m_afRotSpeed[0] = 35;
	m_afRotSpeed[1] = 20;
	m_afRotSpeed[2] = 10;
	// Setup flash effect
	m_afFlashColor[0] = 1;	// Red component
	m_afFlashColor[1] = 1;	// Green component
	m_afFlashColor[2] = 0;	// Blue component
	m_fFlashPeriod = 3;	// 5 second period.
}

/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	ReadConfigRegistry
// DESCRIPTION:	Load configuration values from the registry.
// PARAMETERS:	None
// RETURN VALUE:None
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
void SSMinimalApp::ReadConfigRegistry()
{
	SSUtlReg Reg;
	if(Reg.Open(REGSTR_PATH_CONFIG))
	{
		m_bRotate = Reg.ReadDWord("Rotate", m_bRotate);
		m_bFlash = Reg.ReadDWord("Flash", m_bFlash);
		Reg.ReadString("TextureFile", m_szTextureFile, 256);
		Reg.Close();
	}
}

/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	WriteConfigRegistry
// DESCRIPTION:	Save the configuration values into the registry.
// PARAMETERS:	None
// RETURN VALUE:None
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
void SSMinimalApp::WriteConfigRegistry()
{
	SSUtlReg Reg;
	if(Reg.Create(REGSTR_PATH_CONFIG))
	{
		Reg.WriteDWord("Rotate", m_bRotate);
		Reg.WriteDWord("Flash", m_bFlash);
		Reg.WriteString("TextureFile", m_szTextureFile);
		Reg.Close();
	}
}

/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	SetTextureFile
// DESCRIPTION:	Set the texture file name.
// PARAMETERS:	(in) a_szName	-	Name of the file (full path).
// RETURN VALUE:None
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
void SSMinimalApp::SetTextureFile(const char *a_szName)
{
	strcpy(m_szTextureFile, a_szName);
}

/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	InitGL
// DESCRIPTION:	Initialize the openGL ressources.
// PARAMETERS:	(in) width	-	With of the window in pixels.
//				(in) height	-	Height of the window in pixels.
// RETURN VALUE: true if success, false otherwise.
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
bool SSMinimalApp::InitGL(int width, int height)
{
	// Load a texture
	if(!m_pTexture)
	{
		m_pTexture = new SSTexture(GetTextureFile());
	}

	glEnable(GL_TEXTURE_2D);							// Enable Texture Mapping ( NEW )
	glShadeModel(GL_SMOOTH);							// Enable Smooth Shading
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);				// Black Background
	glClearDepth(1.0f);									// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);								// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	// Really Nice Perspective Calculations

	glViewport(0,0,width,height);						// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);

	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix

	// We are going to need the clock, so initialize it:
	SSClock::Init();
	m_fLastTime = SSClock::GetTime();
	m_fNextFlash = m_fLastTime + m_fFlashPeriod;
	return true;
}


/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	ReleaseGL
// DESCRIPTION:	This function gets called when the application still has the
//				OpenGL context. It is used to release OpenGL ressources like
//				the textures or display lists. It is important to do this
//				since allocated OpenGL ressource may stay alive in the driver
//				even if the context is dead and this is not good. A well behaved
//				OpenGL screen saver always clean up after himself! :)
// PARAMETERS:	None
// RETURN VALUE:None
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
void SSMinimalApp::ReleaseGL()
{
	// Release the texture from OpenGL
	delete m_pTexture;
	// Set the pointer to NULL so that we don't redelete it
	// by mistake in the destructor.
	m_pTexture = NULL;	
}


/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	Update
// DESCRIPTION:	Called every frame to update the application.
// PARAMETERS:	None
// RETURN VALUE:None
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
void SSMinimalApp::Update()
{
	// Query the time and compute elapsed time since last
	// update of the application.
	float fTime = SSClock::GetTime();
	float fDeltaTime = fTime - m_fLastTime;
	m_fLastTime = fTime;	// Prepare for next time.

	// Update the rotation for each axis.
	for(int i=0; i < 3; i++)
	{
		// Update the rotation of the cube on axe i.
		m_afRot[i] += (m_afRotSpeed[i] * fDeltaTime);	
		// Wrap angles around 360 degrees.
		if(m_afRot[i] > 360) m_afRot[i] -= 360;
	}

	// Update the flashing effect
	if(fTime > m_fNextFlash)
	{	// It is time to flash, so we change the clear color.
		glClearColor(m_afFlashColor[0], m_afFlashColor[1], m_afFlashColor[2], 1);
		m_bFlashed = true;
		// Prepare next flash time.
		m_fNextFlash = fTime + m_fFlashPeriod;
	}
	else
	{
		if(m_bFlashed)
		{	// Reset the clear color to black.
			glClearColor(0, 0, 0, 1);
		}
	}
}

/////////////////////////////////////////////////////////////////////////////////
// FUNCTION:	Display
// DESCRIPTION:	Called every frame to display the scene.
// PARAMETERS:	None
// RETURN VALUE:None
// REMARKS:		None
/////////////////////////////////////////////////////////////////////////////////
void SSMinimalApp::Display()
{
	// Clear The Screen And The Depth Buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	// Reset The View
	glLoadIdentity();									
	glTranslatef(0.0f,0.0f,-5.0f);
	// Apply the rotation to the ModelView matrix.
	if(m_bRotate)
	{
		glRotatef(m_afRot[0], 1.0f, 0.0f, 0.0f);
		glRotatef(m_afRot[1], 0.0f, 1.0f, 0.0f);
		glRotatef(m_afRot[2], 0.0f, 0.0f, 1.0f);
	}
	// Apply the texture
	m_pTexture->Apply();
	// Draw a cube.
	glBegin(GL_QUADS);
		// Front Face
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
		// Back Face
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
		// Top Face
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
		// Bottom Face
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
		// Right face
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
		// Left Face
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
	glEnd();
}
